
; VCF-MW 12 6502 Badge Software
; Hardware design by Lee Hart
; Software design by Daryl Rictor
;
; This software is free to use and modify in any non-commercial application.
; Comericial use is prohibited without expressed, written permission from the author.
;
;-------------------------------
; LED driver for VCF-MW 12      |
;                               |
; by Daryl Rictor  May 17, 2017 |
;-------------------------------
	
Lbuff		=	$2A0		; text storage for LED 32 bytes max
LDbuff		=	$2C0		; descrete LED bit pattern buffer 32 bits max
Lptr		= 	$E2		; LED pointer
Ldig		=	$E3		; digit counter
Lscn		=	$E4		; 16 bit scan speed delay
Lscl		=	$E5		; scroll speed
Lscnc		=	$E6		; scan counter
Lsclc		=	$E7		; scroll counter
LEDchk1		=	$E8		; config checksum 1
LEDchk2		=	$E9		; config checksum 2

LEDIO		=	$f000		; IO address base for LED display

LSCAN		=	$70		; constant scan speed
LSCROLL		=	$10		; constant scroll speed

Ldef 		.byte	"6502 bAdGE For VCF              " ; default message
;
; Config a 5 bit binary counter on the descrete LED's
;
;LDdef		.byte	$80, $90, $81, $91, $82, $92, $83, $93, $84, $94, $85, $95, $86, $96, $87, $97 
;		.byte	$88, $98, $89, $99, $8a, $9a, $8b, $9b, $8c, $9c, $8d, $9d, $8e, $9e, $8f, $9f  
;
; Scan back and forth, aka Knightrider, on the descrete LED's
;
LDdef		.byte	$88, $84, $82, $81, $90, $81, $82, $84, $88, $84, $82, $81, $90, $81, $82, $84
		.byte	$88, $84, $82, $81, $90, $81, $82, $84, $88, $84, $82, $81, $90, $81, $82, $84
		

;----------------------------------------------------------------------------
LED_Init	lda	LEDchk1		; check if we have already initialized the video buffer
		cmp	#$A5		; first checksum byte = $A5
		bne	Linit2		; if not a match, we will initialize the checksums and buffer
		lda	LEDchk2		;
		cmp	#$5A		; second checksum is $5A
		beq	Linit3		; match, initialize just buffer pointers and shift counter 
Linit2		lda	#$A5		; init first checksum
		sta	LEDchk1		;
		lda	#$5A		; init second checksum
		sta	LEDchk2		; 
		ldy	#$1F		; store default display in buffer 32 bytes
Linit1		lda	Ldef,y		; text
		sec			; set carry for subtraction
		sbc	#$20		; adjust ASCII code
		tax			; move to pointer
		lda	FONT_TABLE, x	; get ASCII character font
		sta	Lbuff,y		; save in buffer
		lda	LDdef, y	; get descrete LED
		sta	LDbuff, y	; save in buffer
		dey			; get next character
		bpl	Linit1		; loop until all 32 done 
Linit3		STZ	Lptr		; clear pointer
		lda	#LSCAN
		sta	Lscn		; Set scan speed
		sta	Lscnc		; init counter
		lda	#LSCROLL	; set high byte scroll delay
		sta	Lscl		; save it	
		sta	Lsclc		; clear scroll counter 
		rts			; done

;----------------------------------------------------------------------------
; call this to input a text message and convert it to the font codes
;
SLED_cmd	ldx	#$00		; buffer pointer
Sled1		lda	Buffer, y	; get first character
		cmp	#$0D		; is it EOL
		beq	Sled_eol	; yes
		phx			; save x
		sec			; adjust ASCII
		sbc	#$20		; - 32
		tax			; set pointer
		lda	FONT_TABLE,x	; get font byte
		plx			; restore x
		sta	Lbuff,x		; save font to buffer
		iny			; move input buffer pointer
		inx			; move display buffer pointer
		cpx	#$20		; have we done 32 bytes?
		bne	Sled1		; not yet, get next character	
		lda	#$0D		; load CR code
		sta	Buffer+1	; ensure rest of text is ignored
		rts			; done
Sled_eol	lda	#$80		; LED space chr font byte
Sled2		sta	Lbuff, x	; save to disaply buffer
		inx			; inc dispaly buffer
		cpx	#$20		; fill rest of buffer
		bne	Sled2		;	
		jmp	Monitor		; done, bypass Monitor command processing
					; and just start with a new line

;----------------------------------------------------------------------------
; call this to complete 1 digit scan.
; call repeatedly to refresh entire display and scroll
; This gets called while waiting for user input via the Monitor's IO routines.
;
LED_scan	dec	Lscnc		; decrement scan counter		
		bne	Led_sfin3	; not ready yet
		pha			; save a
		phx			; save x
		phy			; save y	
		ldx	Lscn		; get scan speed
		stx	Lscnc		; reset on-time delay
		beq	Led_sfin2	; disable LED refresh if 0
		ldx	Ldig		; digit counter
		lda	LED_COL, x	; get Column mask
		tay			; save mask in Y
		txa			; get digit counter
		clc			; clear carry for add
		adc	Lptr		; add character pointer
		and	#$1F		; wrap if larger than 32 characters
		tax			; set up pointer to data
		lda	Ldig		; digit counter
		cmp	#$07		; is it the descrete LED's
		beq	Led_sdesc	; go to descrete LED update
		lda	Lbuff, x	; get LED scan code
		bra	Led_sfin	; skip next cmd
Led_sdesc	lda	LDbuff, x	; get descrete LED code
Led_sfin	sta	LEDIO,y		; update LED display
		lda	Ldig		; get LED pointer
		ina			; set to next LED
		and	#$07		; wrap if beyond 7
		sta	Ldig		; save it
		bne	Led_sfin2	; all done
		dec	Lsclc		; dec scroll counter
		bne	Led_sfin2	; not ready yet
		lda	Lscl		; done with scroll delay
		sta	Lsclc		; restore scroll counter	
		beq	Led_sfin2	; if scroll delay =0, don't scroll
		lda	Lptr		; get character pointer
		ina			; scroll one to the left
		and	#$1F		; mask to 32 characters
		sta	Lptr		; save it
Led_sfin2	ply			; restore y
		plx			; restore x
		pla			; restore a
Led_sfin3	rts			; done
;----------------------------------------------------------------------------
